﻿using System;
using System.Collections.Concurrent;
using System.Text;

using RabbitMQ.Client;
using RabbitMQ.Client.Events;

namespace TestConsumer
{
    /// <summary>
    /// RPC 클라이언트
    /// </summary>
    public class RPCClient
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 연결
        /// </summary>
        private readonly IConnection connection;

        /// <summary>
        /// 모델
        /// </summary>
        private readonly IModel model;

        /// <summary>
        /// 응답 큐 명칭
        /// </summary>
        private readonly string replyQueueName;

        /// <summary>
        /// 이벤트 기본 소비자
        /// </summary>
        private readonly EventingBasicConsumer eventingBasicConsumer;

        /// <summary>
        /// 기본 속성
        /// </summary>
        private readonly IBasicProperties basicProperties;

        /// <summary>
        /// 차단 컬렉션
        /// </summary>
        private readonly BlockingCollection<string> blockingCollection = new BlockingCollection<string>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RPCClient()

        /// <summary>
        /// 생성자
        /// </summary>
        public RPCClient()
        {
            ConnectionFactory factory = new ConnectionFactory() { HostName = "localhost" };

            this.connection = factory.CreateConnection();

            this.model = this.connection.CreateModel();

            this.replyQueueName = this.model.QueueDeclare().QueueName;

            this.eventingBasicConsumer = new EventingBasicConsumer(this.model);

            this.basicProperties = this.model.CreateBasicProperties();

            string correlationID = Guid.NewGuid().ToString();

            this.basicProperties.CorrelationId = correlationID;
            this.basicProperties.ReplyTo       = this.replyQueueName;

            this.eventingBasicConsumer.Received += (sender, e) =>
            {
                byte[] responseMessageArray = e.Body;

                string responseMessage = Encoding.UTF8.GetString(responseMessageArray);

                if(e.BasicProperties.CorrelationId == correlationID)
                {
                    blockingCollection.Add(responseMessage);
                }
            };
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 피보나치 수열 구하기 - GetFibonacciSequence(value)

        /// <summary>
        /// 피보나치 수열 구하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>피보나치 수열</returns>
        public long GetFibonacciSequence(int value)
        {
            string message = value.ToString();

            byte[] messageByteArray = Encoding.UTF8.GetBytes(message);

            this.model.BasicPublish
            (
                exchange        : "",
                routingKey      : "rpc_queue",
                basicProperties : this.basicProperties,
                body            : messageByteArray
            );

            this.model.BasicConsume
            (
                consumer : eventingBasicConsumer,
                queue    : this.replyQueueName,
                autoAck  : true
            );

            string responseMessage = blockingCollection.Take();

            return Convert.ToInt64(responseMessage);
        }

        #endregion

        #region 닫기 - Close()

        /// <summary>
        /// 닫기
        /// </summary>
        public void Close()
        {
            this.connection.Close();
        }

        #endregion
    }
}