﻿using System;
using System.Text;

using RabbitMQ.Client;
using RabbitMQ.Client.Events;

namespace TestProducer
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        /// <param name="argumentArray">인자 배열</param>
        private static void Main(string[] argumentArray)
        {
            ConnectionFactory factory = new ConnectionFactory() { HostName = "localhost" };

            using(IConnection connection = factory.CreateConnection())
            {
                using(IModel model = connection.CreateModel())
                {
                    model.QueueDeclare
                    (
                        queue      : "rpc_queue",
                        durable    : false,
                        exclusive  : false,
                        autoDelete : false,
                        arguments  : null
                    );

                    model.BasicQos(0, 1, false);

                    EventingBasicConsumer consumer = new EventingBasicConsumer(model);

                    model.BasicConsume
                    (
                        queue    : "rpc_queue",
                        autoAck  : false,
                        consumer : consumer
                    );

                    WriteLog("RPC 요청을 대기하고 있습니다.");

                    consumer.Received += (sender, e) =>
                    {
                        string response = null;

                        byte[] requestArray = e.Body;

                        IBasicProperties requestBasicProperties = e.BasicProperties;

                        IBasicProperties replyBasicProperties = model.CreateBasicProperties();

                        replyBasicProperties.CorrelationId = requestBasicProperties.CorrelationId;

                        try
                        {
                            string requestMessage = Encoding.UTF8.GetString(requestArray);

                            int value = int.Parse(requestMessage);

                            WriteLog("{0}의 피보나치 수열 계산을 시작합니다.", requestMessage);

                            int result = GetFibonacciSequence(value);

                            response = result.ToString();
                        }
                        catch(Exception exception)
                        {
                            WriteLog("예외가 발생했습니다 :\n{0}", exception.Message);

                            response = "";
                        }
                        finally
                        {
                            byte[] responseByteArray = Encoding.UTF8.GetBytes(response);

                            model.BasicPublish
                            (
                                exchange        : "",
                                routingKey      : requestBasicProperties.ReplyTo,
                                basicProperties : replyBasicProperties,
                                body            : responseByteArray
                            );

                            model.BasicAck
                            (
                                deliveryTag : e.DeliveryTag,
                                multiple    : false
                            );
                        }
                    };

                    Console.WriteLine("프로그램을 종료하려면 아무 키나 눌러 주시기 바랍니다.");

                    Console.ReadKey(true);
                }
            }
        }

        #endregion

        #region 로그 작성하기 - WriteLog(format, parameterArray)

        /// <summary>
        /// 로그 작성하기
        /// </summary>
        /// <param name="format">포맷 문자열</param>
        /// <param name="parameterArray">매개 변수 배열</param>
        private static void WriteLog(string format, params object[] parameterArray)
        {
            string message;

            if(parameterArray.Length == 0)
            {
                message = format;
            }
            else
            {
                message = string.Format(format, parameterArray);
            }

            string log = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), message);

            Console.WriteLine(log);
        }

        #endregion
        #region 피보나치 수열 구하기 - GetFibonacciSequence(sourceValue)

        /// <summary>
        /// 피보나치 수열 구하기
        /// </summary>
        /// <param name="sourceValue">소스 값</param>
        /// <returns>피보나치 수열</returns>
        private static int GetFibonacciSequence(int sourceValue)
        {
            if(sourceValue == 0 || sourceValue == 1)
            {
                return sourceValue;
            }

            return GetFibonacciSequence(sourceValue - 1) + GetFibonacciSequence(sourceValue - 2);
        }

        #endregion
    }
}